import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

public class MakeSubwayData {
	static String naverUrl = "http://map.naver.com/local/siteview.nhn?stationId=%s&theme=subway";
	
	static String strCID; // 지역코드
	static String strLID; // 노선코드
	static String strSID; // 역코드
	static String strStationName; // 역명
	static String strStationLine; // 호선
	static String strStationURL; // URL(m.seoul.go.kr)
	static String strpreStation; // 이전 역
	static String strnextStation; // 다음 역
	static String strStationInfo; // 역정보(전화번호, 주소)
	static String strStaionExit; // 출구정보
	static String strStationTime; // 첫차, 막차정보
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		Document doc;
		StationList stationList = new StationList();
		
		try {
			Class.forName("org.sqlite.JDBC");
			Connection conn = null;
			Statement stat;
			PreparedStatement prep = null;

			conn = DriverManager.getConnection("jdbc:sqlite:subway.db");
			stat = conn.createStatement();
			stat.executeUpdate("drop table if exists Station;");
			stat.executeUpdate("CREATE TABLE Station(StationName, StationLine, StationURL, preStation, nextStation, StationInfo, StationExit, StationTime, primary key (StationLine, stationName, StationURL));");
			prep = conn
					.prepareStatement("INSERT into Station values (?, ?, ?, ?, ?, ?, ?, ?);");

			/**
			 * 자료수집
			 */
			// ArrayList 모든역
			for (StationList list : stationList.getStationList()) {

				// 초기화
				strpreStation = ""; // 이전 역
				strnextStation = ""; // 다음 역
				strStaionExit = ""; // 출구정보
				strStationURL = ""; // URL(m.seoul.go.kr)
				strStationInfo = ""; // 역정보(전화번호, 주소)
				strStationTime = ""; // 첫차, 막차정보
				strStationName = list.getName();
				strStationLine = list.getLine();
				
				doc = Jsoup.connect(String.format(naverUrl, list.getCode())).get();

				strpreStation = doc.select(".sway_prev .in").text().replace(" | ", ", ").trim();
				strnextStation = doc.select(".sway_next .in").text().replace(" | ", ", ").trim();
				
				/**
				 * 역 정보
				 */
				getStationInfo(doc);

				/**
				 * 출구 정보
				 */
				getExitInfo(doc);

				/**
				 * 첫차, 막차 스케줄
				 */
				getScheduleInfo(doc);

				/**
				 * 모바일서울 URL
				 */
				getMobileSeoulURL();

				// DB 추가.
				prep.setString(1, strStationName);
				prep.setString(2, strStationLine);
				prep.setString(3, strStationURL);
				prep.setString(4, strpreStation);
				prep.setString(5, strnextStation);
				prep.setString(6, strStationInfo);
				prep.setString(7, strStaionExit);
				prep.setString(8, strStationTime);

				prep.addBatch();

				// 데이터가 잘 수집되었는지 확인하기 위한 Log
				String strLog = String.format("%s, %s, %s, %s, %s, %s",
						strStationName, strStationLine, strStationURL,
						strpreStation, strnextStation, strStationInfo);
				System.out.println(strLog);
	
				Thread.sleep(100);
			} // DB와 연결이 끊어지지 않았다면 데이터를 삽입한다.
			
			if (conn != null) {
				conn.setAutoCommit(false);
				prep.executeBatch();
				conn.setAutoCommit(true);
			}
		} catch (InterruptedException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * 역 정보 (주소, 전화번호)
	 */
	private static void getStationInfo(Document doc) {
		String strAddress = "주소 : " + doc.select(".local_info_detail a").text();
		String strTel = "전화번호 : " + doc.select(".tel").text();
		
		strStationInfo = String.format("%s\n%s", strAddress, strTel);
		
//		System.out.println(strStationInfo);
	}
	
	/**
	 * 출구 정보
	 */
	private static void getExitInfo(Document doc) {
		Elements elements = doc.select(".section_sway_exitbus_tb tr");
		for (Element element : elements) {
			if (element.hasClass("tbs3_only") || element.hasClass("tbs3_top")) {
				strStaionExit += element.select("th").text() + " : " + element.select("td").text() + "//"; 
			}
		}

//		System.out.println(strStaionExit);
	}
	
	/**
	 * 첫차, 막차 정보
	 */
	private static void getScheduleInfo(Document doc) {
		/*
		 * <평일>
		 * 05:10(성북),05:57(청량리),###05:41(신창),06:11(천안),06:22(서동탄),###//22:13(성북),22:45(동묘앞),23:16(청량리),23:49(구로),###23:08(신창),23:38(천안),24:04(서동탄),24:21(병점),###/>
		 * <토요일>
		 * 05:11(성북),05:33(청량리),###05:41(신창),06:22(서동탄),###//22:03(청량리),22:29(성북),22:57(동묘앞),23:18(서울역),23:33(구로),###23:08(신창),23:29(천안),23:44(서동탄),24:20(병점),###/>
		 * <공휴일>
		 * 05:11(성북),05:33(청량리),###05:41(신창),06:22(서동탄),###//22:03(청량리),22:29(성북),22:57(동묘앞),23:18(서울역),23:33(구로),###23:08(신창),23:29(천안),23:44(서동탄),24:20(병점),###/>
		 */
		
		boolean isFirst = false;
		
		String strTimeWorkdayFirst = "";
		String strTimeWorkdayLast = "";
		String strTimeSaturdayFirst = "";
		String strTimeSaturdayLast = "";
		String strTimeSundayFirst = "";
		String strTimeSundayLast = "";
		
		Elements elements = doc.select(".section_sway_time_tb .tbs1.tbs2");
		
		for (Element element : elements) {
			Elements schedules = element.select("tbody tr"); 
			
			for (Element schedule : schedules) {
				if (schedule.select(".thf").text().equalsIgnoreCase("첫차")) {
					isFirst = true;
				} else if (schedule.select(".thf").text().equalsIgnoreCase("막차")) {
					isFirst = false;
				}
				
				String destination = schedule.select(".t").text();
				Elements times = schedule.select("td");
				
				for (int j = 1; j < times.size(); j++) {
					if (times.get(j).text().equals("-")) {
						continue;
					}
					
					String item = String.format("%s(%s),", times.get(j).text(), destination);
					if (isFirst) {
						if (j == 1) {
							strTimeWorkdayFirst += item;
						} else if (j == 2) {
							strTimeSaturdayFirst += item;
						} else if (j == 3) {
							strTimeSundayFirst += item;
						}
					} else {
						if (j == 1) {
							strTimeWorkdayLast += item;
						} else if (j == 2) {
							strTimeSaturdayLast += item;
						} else if (j == 3) {
							strTimeSundayLast += item;
						}
					}
				}
			}
			
			// 상행과 하행 구분
			strTimeWorkdayFirst += "###";
			strTimeSaturdayFirst += "###";
			strTimeSundayFirst += "###";
			strTimeWorkdayLast += "###";
			strTimeSaturdayLast += "###";
			strTimeSundayLast += "###";
		}
		
		strStationTime = "<평일>\n" + strTimeWorkdayFirst + "//"
				+ strTimeWorkdayLast + "/>\n" + "<토요일>\n"
				+ strTimeSaturdayFirst + "//" + strTimeSaturdayLast + "/>\n"
				+ "<공휴일>\n" + strTimeSundayFirst + "//" + strTimeSundayLast
				+ "/>\n";
		
//		System.out.println(strStationTime);
	}
	
	/**
	 * 모바일서울 URL
	 */
	private static void getMobileSeoulURL() {
		String strURL = "";
		String strTempStationName = "";

		// 총신대입구(이수)의 경우가 있을 수 있다.
		Pattern pattern = Pattern.compile("(.+)\\((.+)\\)");
		Matcher matches = pattern.matcher(strStationName);

		if (matches.find()) {
			strTempStationName = matches.group(1);
		} else {
			strTempStationName = strStationName;
		}

		// '서울역'의 경우 '서울' 로 검색한다.
		int nIndex = strStationName.lastIndexOf("역");

		if (nIndex == strStationName.length() - 1) {
			strTempStationName = strStationName.substring(0, nIndex);
		} else if (strStationName.indexOf(" ") > 0) {
			// 역명에 공백이 있으면 제거.
			strTempStationName = strStationName.replace(" ", "");
		}

		try {
			strURL = "http://m.seoul.go.kr/traffic/SubInfoSearchList.do?subSearch="
					+ URLEncoder.encode(strTempStationName, "EUC-KR")
					+ "&flag=4";
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		try {
			Document doc = Jsoup.connect(strURL).get();
			
			for (Element element : doc.select(".lst li")) {
				
				String strTempLine = CommonFunction
						.getStrStationLine(element.select("div").attr("class"));
				String strTempName = element.select("div").text();
				String strName2 = "";
				
				pattern = Pattern.compile("(.+) (.+)");
				matches = pattern.matcher(strTempName);

				while (matches.find()) {
					strTempName = matches.group(2);

					if (strTempName.length() != strTempStationName.length()) {

						// 총신대입구(이수)의 경우가 있을 수 있다.
						Pattern namePattern = Pattern.compile("(.+)\\((.+)\\)");
						Matcher nameMatches = namePattern.matcher(strTempName);

						if (nameMatches.find()) {
							strTempName = nameMatches.group(1);
							strName2 = nameMatches.group(2);
						} else {
							namePattern = Pattern.compile("(.+) (.+)");
							nameMatches = namePattern.matcher(strTempName);

							if (nameMatches.find()) {
								strTempName = nameMatches.group(1);
							}
						}
					}
				}

				String strTempStationLine = strStationLine;
				
				// 역명과 호선이 일치 할 경우만 URL을 저장한다.
				if (strTempStationLine.compareTo(strTempLine) == 0
						&& strTempStationName.compareTo(strTempName) == 0) {
					strStationURL = element.select("a").attr("href")
							.replace("&amp;", "&");
					break;
				} else if (strTempStationLine.compareTo(strTempLine) == 0
						&& strTempStationName.compareTo(strName2) == 0) {
					strStationURL = element.select("a").attr("href")
							.replace("&amp;", "&");
					break;
				} else {
					continue;
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}